/*
 * bound.c
 *
 * Author:  Shuro Shindo <shu@pegasus.rim.or.jp>
 * Created: Mar. 31, 1996
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include "vga.h"
#include "vgagl.h"

#define	BlockWidth	30
#define	BlockHeight	30
#define	DeltaX		1
#define DeltaY 		1

/* If moving speed is too slow, decrease Delay */
#define Delay		4000


/* Following macro variables are defined in vgagl.h.
   WIDTH, HEIGHT, COLORS, BYTESPERPIXEL
*/

typedef struct point_s {
	int	x;
	int	y;
}	point_t;

void handler( int signal )
{
	vga_setmode( TEXT );
	exit( -1 );
}

int initialization( void )
{
	int	ret;
	int	mode;

	ret = vga_init();
	if ( ret != 0 ) {
		fprintf( stderr, "Can't initialize vga." );
		return( 1 );
	}
	ret = vga_setmode( G640x480x256 );
	if ( ret != 0 ) {
		ret = vga_setmode( G640x480x16 );
		if ( ret != 0 ) {
			fprintf( stderr, "Can't set graphics mode to 640x480x[16|256]." );
			return( 1 );
		}
	}
	mode = vga_getcurrentmode();
	gl_setcontextvga( mode );
	return( 0 );
}

void draw( point_t *p, int w, int h, int color )
{
	int	i;

	gl_fillbox( p->x, p->y, w, h, color );

	/* timing adjust */
	for ( i = 0; i < Delay; ++i ) {
		vga_waitretrace();
	}
}

void calcpos( point_t *p, int w, int h, int *dx, int *dy )
{
	if ( *dx >= 0 ) {
		if ( p->x + w >= WIDTH ) {
			*dx = - DeltaX;
		}
	}
	else {
		if ( p->x <= 0 ) {
			*dx = DeltaX;
		}
	}
	if ( *dy >= 0 ) {
		if ( p->y + h >= HEIGHT ) {
			*dy = - DeltaY;
		}
	}
	else {
		if ( p->y <= 0 ) {
			*dy = DeltaY;
		}
	}
}

void main( void )
{
	point_t	p1 = { 0, 0 };
	int	width = BlockWidth;
	int	height = BlockHeight;
	int	dx = DeltaX;
	int	dy = DeltaY;

	int	ret;
	char	*buf;
	int	color = 1;	/* blue */


	/* initialization */
	signal( SIGQUIT, handler );

	ret = initialization();
	if ( ret != 0 ) {
		exit( 1 );
	}
	printf( "width:%d, height:%d, colors:%d, bytes/pixel:%d\n", 
		WIDTH, HEIGHT, COLORS, BYTESPERPIXEL );

	buf = malloc( width * height * BYTESPERPIXEL );
	if ( buf == NULL ) {	/* not enough memory */
		exit( 2 );
	}

	for ( ; ; ) {

		/* save */
		gl_getbox( p1.x, p1.y, width, height, buf );

		/* draw */
		draw( &p1, width, height, color );
/*		color++; color %= COLORS;*/

		/* calc new position */
		calcpos( &p1, width, height, &dx, &dy );

		/* restore */
		gl_putbox( p1.x, p1.y, width, height, buf );

		/* set new position */
		p1.x += dx;
		p1.y += dy;
	}
}
